@@ -16,9 +16,15 @@ void print_all_menu_items(FILE* rsp) {
                                                       kCGNullWindowID        );
   int window_count = CFArrayGetCount(window_list);
 
-  fprintf(rsp, "[\n");
-  int counter = 0;
+  float x_pos[window_count];
+  char* owner[window_count];
+  char* name[window_count];
+  memset(owner, 0, sizeof(owner));
+  memset(name, 0, sizeof(name));
+
+  int item_count = 0;
   for (int i = 0; i < window_count; ++i) {
+    x_pos[i] = -9999.f;
     CFDictionaryRef dictionary = CFArrayGetValueAtIndex(window_list, i);
     if (!dictionary) continue;
 
@@ -30,7 +36,11 @@ void print_all_menu_items(FILE* rsp) {
 
     CFStringRef name_ref = CFDictionaryGetValue(dictionary, kCGWindowName);
     CFNumberRef layer_ref = CFDictionaryGetValue(dictionary, kCGWindowLayer);
-    if (!name_ref || !owner_ref || !owner_pid_ref || !layer_ref) continue;
+    CFDictionaryRef bounds_ref = CFDictionaryGetValue(dictionary,
+                                                      kCGWindowBounds);
+
+    if (!name_ref || !owner_ref || !owner_pid_ref || !layer_ref || !bounds_ref)
+      continue;
 
     long long int layer = 0;
     CFNumberGetValue(layer_ref, CFNumberGetType(layer_ref), &layer);
@@ -40,20 +50,43 @@ void print_all_menu_items(FILE* rsp) {
                      &owner_pid                     );
 
     if (layer != MENUBAR_LAYER) continue;
-    char* owner = cfstring_copy(owner_ref);
-    char* name = cfstring_copy(name_ref);
+    CGRect bounds = CGRectNull;
+    if (!CGRectMakeWithDictionaryRepresentation(bounds_ref, &bounds)) continue;
+    owner[item_count] = cfstring_copy(owner_ref);
+    name[item_count] = cfstring_copy(name_ref);
+    x_pos[item_count++] = bounds.origin.x;
+  }
 
-    if (strcmp(name, "") != 0) {
-      if (counter++ > 0) {
-        fprintf(rsp, ", \n");
+  if (item_count > 0) {
+    fprintf(rsp, "[\n");
+    int counter = 0;
+    for (int i = 0; i < item_count; i++) {
+      float current_pos = x_pos[0];
+      uint32_t current_pos_id = 0;
+      for (int j = 0; j < window_count; j++) {
+        if (!name[j] || !owner[j]) continue;
+        if (x_pos[j] > current_pos) {
+          current_pos = x_pos[j];
+          current_pos_id = j;
+        }
       }
-      fprintf(rsp, "\t\"%s,%s\"", owner, name);
+
+      if (!name[current_pos_id] || !owner[current_pos_id]) continue;
+      if (strcmp(name[current_pos_id], "") != 0) {
+        if (counter++ > 0) {
+          fprintf(rsp, ", \n");
+        }
+        fprintf(rsp, "\t\"%s,%s\"", owner[current_pos_id],
+                                    name[current_pos_id]  );
+      }
+      x_pos[current_pos_id] = -9999.f;
     }
-
-    free(owner);
-    free(name);
+    fprintf(rsp, "\n]\n");
+    for (int i = 0; i < window_count; i++) {
+      if (owner[i]) free(owner[i]);
+      if (name[i]) free(name[i]);
+    }
   }
-  fprintf(rsp, "\n]\n");
   CFRelease(window_list);
 }
 
@@ -78,7 +111,7 @@ void alias_init(struct alias* alias) {
   alias->name = NULL;
   alias->owner = NULL;
   alias->color_override = false;
-  alias->color = rgba_color_from_hex(0xffff0000);
+  color_init(&alias->color, 0xffff0000);
   alias->update_frequency = 1;
   alias->counter = 0;
 
@@ -86,23 +119,6 @@ void alias_init(struct alias* alias) {
   image_init(&alias->image);
 }
 
-void alias_setup(struct alias* alias, char* owner, char* name) {
-  alias->name = name;
-  alias->owner = owner;
-  alias_get_permission(alias);
-  alias_update_image(alias);
-}
-
-uint32_t alias_get_length(struct alias* alias) {
-  if (alias->image.image_ref) return alias->image.bounds.size.width;
-  return 0;
-}
-
-uint32_t alias_get_height(struct alias* alias) {
-  if (alias->image.image_ref) return alias->image.bounds.size.height;
-  return 0;
-}
-
 static void alias_find_window(struct alias* alias) {
   CFArrayRef window_list = CGWindowListCopyWindowInfo(kCGWindowListOptionAll,
                                                       kCGNullWindowID        );
@@ -171,27 +187,51 @@ static void alias_find_window(struct alias* alias) {
   CFRelease(window_list);
 }
 
-bool alias_update_image(struct alias* alias) {
+static bool alias_update_image(struct alias* alias, bool forced) {
   if (alias->window.id == 0) alias_find_window(alias);
   if (alias->window.id == 0) return false;
 
-  CGImageRef image_ref = window_capture(&alias->window);
+  bool disabled = false;
+  CGImageRef image_ref = window_capture(&alias->window, &disabled);
 
   if (!image_ref) {
-    window_init(&alias->window);
+    if (!disabled) {
+      alias->window.id = 0;
+      image_destroy(&alias->image);
+    }
     return false;
   }
 
-  return image_set_image(&alias->image, image_ref, alias->window.frame, false);
+  return image_set_image(&alias->image,
+                         image_ref,
+                         alias->window.frame,
+                         forced              );
 }
 
-bool alias_update(struct alias* alias) {
+void alias_setup(struct alias* alias, char* owner, char* name) {
+  alias->name = name;
+  alias->owner = owner;
+  alias_get_permission(alias);
+  alias_update_image(alias, true);
+}
+
+uint32_t alias_get_length(struct alias* alias) {
+  if (alias->image.image_ref) return alias->image.bounds.size.width;
+  return 0;
+}
+
+uint32_t alias_get_height(struct alias* alias) {
+  if (alias->image.image_ref) return alias->image.bounds.size.height;
+  return 0;
+}
+
+bool alias_update(struct alias* alias, bool forced) {
   if (alias->update_frequency == 0) return false;
 
   alias->counter++;
-  if (alias->counter >= alias->update_frequency) {
+  if (forced || alias->counter >= alias->update_frequency) {
     alias->counter = 0;
-    if (alias_update_image(alias)) {
+    if (alias_update_image(alias, forced)) {
       return true;
     }
   }
@@ -239,14 +279,22 @@ bool alias_parse_sub_domain(struct alias* alias, FILE* rsp, struct token propert
                                      rsp,
                                      entry,
                                      message              );
+    else if (token_equals(subdom, SUB_DOMAIN_COLOR)) {
+      bool changed = !alias->color_override;
+      alias->color_override = true;
+      return color_parse_sub_domain(&alias->color, rsp, entry, message)
+            || changed;
+    }
     else {
       respond(rsp, "[!] Alias: Invalid subdomain '%s'\n", subdom.text);
     }
   }
   else if (token_equals(property, PROPERTY_COLOR)) {
-    alias->color = rgba_color_from_hex(token_to_uint32t(get_token(&message)));
+    color_set_hex(&alias->color, token_to_uint32t(get_token(&message)));
     alias->color_override = true;
     return true;
+  } else if (token_equals(property, PROPERTY_SCALE)) {
+    return image_set_scale(&alias->image, token_to_float(get_token(&message)));
   } else if (token_equals(property, PROPERTY_UPDATE_FREQ)) {
     alias->update_frequency = token_to_uint32t(get_token(&message));
     return false;
