@@ -19,19 +19,18 @@ struct alias {
   struct window window;
 
   bool color_override;
-  struct rgba_color color;
+  struct color color;
   struct image image;
 };
 
 void alias_init(struct alias* alias);
 void alias_setup(struct alias* alias, char* owner, char* name);
-bool alias_update_image(struct alias* alias);
 uint32_t alias_get_length(struct alias* alias);
 uint32_t alias_get_height(struct alias* alias);
 
 void alias_calculate_bounds(struct alias* alias, uint32_t x, uint32_t y);
 void alias_draw(struct alias* alias, CGContextRef context);
-bool alias_update(struct alias* alias);
+bool alias_update(struct alias* alias, bool forced);
 void alias_destroy(struct alias* alias);
 
 void print_all_menu_items(FILE* rsp);
