@@ -1,4 +1,5 @@
 #pragma once
+#include <CoreVideo/CoreVideo.h>
 #include "misc/helpers.h"
 
 extern struct bar_manager g_bar_manager;
@@ -56,7 +57,7 @@ extern struct bar_manager g_bar_manager;
                     t, \
                     g_bar_manager.animator.duration, \
                     g_bar_manager.animator.interp_function ); \
-    animation->seperate_bytes = true; \
+    animation->separate_bytes = true; \
     animator_add(&g_bar_manager.animator, animation); \
   } else { \
     needs_refresh = animator_cancel(&g_bar_manager.animator, (void*)o, (bool (*)(void*, int))&f); \
@@ -74,19 +75,21 @@ typedef ANIMATION_FUNCTION(animation_function);
 #define INTERP_FUNCTION_QUADRATIC 'q'
 #define INTERP_FUNCTION_SIN       's'
 #define INTERP_FUNCTION_TANH      't'
+#define INTERP_FUNCTION_CIRC      'c'
 #define INTERP_FUNCTION_BOUNCE    'b'
 #define INTERP_FUNCTION_EXP       'e'
 #define INTERP_FUNCTION_OVERSHOOT 'o'
 
 
 struct animation {
-  bool seperate_bytes;
+  bool separate_bytes;
   bool as_float;
   bool locked;
+  bool finished;
+  bool waiting;
 
-  uint32_t duration;
-  uint32_t counter;
-  uint32_t offset;
+  uint64_t initial_time;
+  double duration;
 
   int initial_value;
   int final_value;
@@ -94,19 +97,18 @@ struct animation {
 
   void* target;
   animator_function* update_function;
+
+  struct animation* next;
+  struct animation* previous;
 };
 
 struct animation* animation_create();
 void animation_setup(struct animation* animation, void* target, animator_function* update_function, int initial_value, int final_value, uint32_t duration, char interp_function);
 
-extern struct event_loop g_event_loop;
-
-#define ANIMATOR_CALLBACK(name) void name(CFRunLoopTimerRef timer, void *context)
-typedef ANIMATOR_CALLBACK(animator_callback);
-
 struct animator {
-  CFRunLoopTimerRef clock;
+  CVDisplayLinkRef display_link;
 
+  double clock;
   uint32_t interp_function;
   uint32_t duration;
   struct animation** animations;
@@ -115,7 +117,13 @@ struct animator {
 
 void animator_init(struct animator* animator);
 void animator_add(struct animator* animator, struct animation* animation);
+
 bool animator_cancel(struct animator* animator, void* target, animator_function* function);
 void animator_cancel_locked(struct animator* animator, void* target, animator_function* function);
-bool animator_update(struct animator* animator);
+
+bool animator_update(struct animator* animator, uint64_t time);
 void animator_lock(struct animator* animator);
+void animator_destroy(struct animator* animator);
+
+void animator_renew_display_link(struct animator* animator);
+void animator_destroy_display_link(struct animator* animator);
