@@ -1,11 +1,20 @@
 #include "bar.h"
 #include "bar_manager.h"
 #include "event.h"
-#include "event_loop.h"
 #include "display.h"
 #include "misc/helpers.h"
 #include "window.h"
 
+#define MAX_RENDER_THREADS 10
+static pthread_t g_render_threads[MAX_RENDER_THREADS];
+static uint32_t g_used_threads = 0;
+
+void join_render_threads() {
+  for (int i = 0; i < g_used_threads; i++)
+    pthread_join(g_render_threads[i], NULL);
+  g_used_threads = 0;
+}
+
 bool bar_draws_item(struct bar* bar, struct bar_item* bar_item) {
     if (!bar_item->drawing || !bar->shown || bar->hidden) return false;
 
@@ -138,24 +147,21 @@ static void bar_check_for_clip_updates(struct bar* bar) {
   }
 }
 
-void bar_draw(struct bar* bar) {
+void bar_draw(struct bar* bar, bool forced, bool threaded) {
   if (bar->sid < 1 || bar->adid < 1) return;
 
   if (g_bar_manager.might_need_clipping)
     bar_check_for_clip_updates(bar);
 
   if (g_bar_manager.bar_needs_update) {
-    draw_rect(bar->window.context,
-              bar->window.frame,
-              &g_bar_manager.background.color,
-              g_bar_manager.background.corner_radius,
-              g_bar_manager.background.border_width,
-              &g_bar_manager.background.border_color,
-              true                                   );
-
-    if (g_bar_manager.background.image.enabled) {
-      image_draw(&g_bar_manager.background.image, bar->window.context);
-    }
+    struct background background = g_bar_manager.background;
+    background.bounds = bar->window.frame;
+    background.bounds.origin.y -= background.y_offset;
+    background.shadow.enabled = false;
+    background.enabled = true;
+    windows_freeze();
+    CGContextClearRect(bar->window.context, bar->window.frame);
+    background_draw(&background, bar->window.context);
   }
 
   for (int i = 0; i < g_bar_manager.bar_item_count; i++) {
@@ -182,21 +188,37 @@ void bar_draw(struct bar* bar) {
                         bar                                     );
     }
 
-    if (!window_apply_frame(window) && !bar_item->needs_update) continue;
+    if (!window_apply_frame(window, forced) && !bar_item->needs_update)
+      continue;
 
     if (bar_item->update_mask & UPDATE_MOUSE_ENTERED
         || bar_item->update_mask & UPDATE_MOUSE_EXITED) {
       window_assign_mouse_tracking_area(window, window->frame);
     }
 
-    CGContextClearRect(window->context, window->frame);
-
-    bar_item_draw(bar_item, window->context);
-    CGContextFlush(window->context);
+    windows_freeze();
+    if (threaded && g_used_threads < MAX_RENDER_THREADS) {
+      uint32_t thread_id = g_used_threads++;
+      struct draw_item_payload {
+        struct window* window;
+        struct bar_item bar_item;
+      }* context = malloc(sizeof(struct draw_item_payload));
+      // We need to perform a shallow copy of the item here because
+      // the cached bounds will be invalidated when drawing the next bar.
+      context->bar_item = *bar_item;
+      context->window = window;
+      pthread_create(&g_render_threads[thread_id],
+                     NULL,
+                     draw_item_proc,
+                     context        );
+    } else {
+      CGContextClearRect(window->context, window->frame);
+      bar_item_draw(bar_item, window->context);
+      CGContextFlush(window->context);
+    }
   }
 
-  if (g_bar_manager.bar_needs_update)
-    CGContextFlush(bar->window.context);
+  if (g_bar_manager.bar_needs_update) CGContextFlush(bar->window.context);
 }
 
 static void bar_calculate_bounds_top_bottom(struct bar* bar) {
@@ -252,7 +274,6 @@ static void bar_calculate_bounds_top_bottom(struct bar* bar) {
 
     if (bar_item->position == POSITION_RIGHT
         || bar_item->position == POSITION_CENTER_LEFT) {
-
       *next_position = min(*next_position - bar_item_display_length
                            - bar_item->background.padding_right,
                            bar->window.frame.size.width
@@ -291,7 +312,7 @@ static void bar_calculate_bounds_top_bottom(struct bar* bar) {
                         ? bar_item_display_length
                           + bar_item->background.padding_right
                           - bar_item->custom_width
-                        : - bar_item->background.padding_left;
+                        : (- bar_item->background.padding_left);
     } else {
       *next_position += bar_item->has_const_width
                         ? bar_item->custom_width
@@ -314,6 +335,9 @@ static void bar_calculate_bounds_top_bottom(struct bar* bar) {
     window_set_frame(bar_item_get_window(bar_item->group->members[0],
                                          bar->adid                   ),
                      bar_item->group->bounds                           );
+
+    if (bar_item->popup.drawing)
+      bar_calculate_popup_anchor_for_bar_item(bar, bar_item);
   }
 }
 
@@ -486,9 +510,12 @@ static CGRect bar_get_frame(struct bar *bar) {
 }
 
 void bar_resize(struct bar* bar) {
-  if (bar->hidden) return;
+  if (bar->hidden || !bar->shown) {
+    window_move(&bar->window, g_nirvana);
+    return;
+  };
   window_set_frame(&bar->window, bar_get_frame(bar));
-  if (window_apply_frame(&bar->window)) {
+  if (window_apply_frame(&bar->window, false)) {
     window_assign_mouse_tracking_area(&bar->window, bar->window.frame);
     g_bar_manager.bar_needs_update = true;
   }
@@ -531,7 +558,7 @@ struct bar *bar_create(uint32_t did) {
   bar->did = did;
   bar->dsid = display_space_id(did);
   bar->sid = mission_control_index(bar->dsid);
-  bar->shown = true;
+  bar->shown = SLSSpaceGetType(g_connection, bar->dsid) != 4;
   g_bar_manager.bar_needs_update = true;
   bar_create_window(bar);
   return bar;
