@@ -1,9 +1,10 @@
 #include "bar_item.h"
 #include "bar_manager.h"
-#include "event_loop.h"
 #include "event.h"
 #include "volume.h"
 #include "power.h"
+#include "media.h"
+#include "app_windows.h"
 
 struct bar_item* bar_item_create() {
   struct bar_item* bar_item = malloc(sizeof(struct bar_item));
@@ -18,7 +19,6 @@ void bar_item_init(struct bar_item* bar_item, struct bar_item* default_item) {
   bar_item->updates = true;
   bar_item->updates_only_when_shown = false;
   bar_item->selected = false;
-  bar_item->mouse_over = false;
   bar_item->ignore_association = false;
   bar_item->overrides_association = false;
   bar_item->counter = 0;
@@ -33,6 +33,8 @@ void bar_item_init(struct bar_item* bar_item, struct bar_item* default_item) {
   bar_item->blur_radius = 0;
   bar_item->event_port = 0;
   bar_item->shadow = false;
+  bar_item->scroll_texts = false;
+  bar_item->mouse_over = false;
 
   bar_item->has_const_width = false;
   bar_item->custom_width = 0;
@@ -106,17 +108,26 @@ void bar_item_reset_associated_bar(struct bar_item* bar_item) {
 }
 
 bool bar_item_update(struct bar_item* bar_item, char* sender, bool forced, struct env_vars* env_vars) {
+  bool is_shown = bar_item_is_shown(bar_item);
+  if (is_shown && bar_item->scroll_texts && (bar_item->counter % 15 == 0)) {
+    text_animate_scroll(&bar_item->icon);
+    text_animate_scroll(&bar_item->label);
+    if (bar_item->type == BAR_COMPONENT_SLIDER)
+      text_animate_scroll(&bar_item->slider.knob);
+  }
+
+  bar_item->counter++;
+
   if ((!bar_item->updates || (bar_item->update_frequency == 0 && !sender))
       && !forced                                                          ) {
     return false;
   }
 
-  bar_item->counter++;
-
   bool scheduled_update_needed = bar_item->update_frequency
                                   <= bar_item->counter;
+
   bool should_update = bar_item->updates_only_when_shown
-                       ? bar_item_is_shown(bar_item)
+                       ? is_shown
                        : true;
 
   if (((scheduled_update_needed || sender) && should_update) || forced) {
@@ -158,17 +169,26 @@ bool bar_item_update(struct bar_item* bar_item, char* sender, bool forced, struc
       free(message);
     }
   }
+
   return false;
 }
 
 void bar_item_needs_update(struct bar_item* bar_item) {
-  if (bar_item->group && bar_item != bar_item->group->members[0]) {
-      bar_item_needs_update(bar_item->group->members[0]);
-  }
-    
   bar_item->needs_update = true;
 }
 
+void bar_item_cancel_drag(struct bar_item* bar_item) {
+  if (bar_item->has_slider) {
+    char perc_str[8];
+    snprintf(perc_str, 8, "%d", bar_item->slider.percentage);
+    env_vars_set(&bar_item->signal_args.env_vars,
+                 string_copy("PERCENTAGE"),
+                 string_copy(perc_str)           );
+
+    slider_cancel_drag(&bar_item->slider);
+  }
+}
+
 void bar_item_on_drag(struct bar_item* bar_item, CGPoint point) {
   if (bar_item->has_slider) {
     if (slider_handle_drag(&bar_item->slider, point)) {
@@ -177,14 +197,30 @@ void bar_item_on_drag(struct bar_item* bar_item, CGPoint point) {
   }
 }
 
-void bar_item_on_click(struct bar_item* bar_item, uint32_t type, uint32_t modifier, CGPoint point) {
+void bar_item_on_click(struct bar_item* bar_item, uint32_t type, uint32_t mouse_button_code, uint32_t modifier, CGPoint point) {
   if (!bar_item) return;
 
-  env_vars_set(&bar_item->signal_args.env_vars,
+  struct env_vars env_vars;
+  env_vars_init(&env_vars);
+  char info_str[256];
+  snprintf(info_str, 256, "{\n"
+                         "\t\"button\": \"%s\",\n"
+                         "\t\"button_code\": %u,\n"
+                         "\t\"modifier\": \"%s\",\n"
+                         "\t\"modfier_code\": %u\n"
+                         "}\n",
+                         get_type_description(type),
+                         mouse_button_code,
+                         get_modifier_description(modifier),
+                         modifier                           );
+
+  env_vars_set(&env_vars, string_copy("INFO"), string_copy(info_str));
+
+  env_vars_set(&env_vars,
                string_copy("BUTTON"),
                string_copy(get_type_description(type)));
 
-  env_vars_set(&bar_item->signal_args.env_vars,
+  env_vars_set(&env_vars,
                string_copy("MODIFIER"),
                string_copy(get_modifier_description(modifier)));
 
@@ -195,33 +231,71 @@ void bar_item_on_click(struct bar_item* bar_item, uint32_t type, uint32_t modifi
         bar_item_needs_update(bar_item);
       }
 
-      char perc_str[8];
-      snprintf(perc_str, 8, "%d", bar_item->slider.percentage);
-      env_vars_set(&bar_item->signal_args.env_vars,
-                   string_copy("PERCENTAGE"),
-                   string_copy(perc_str)           );
-      bar_item->slider.is_dragged = false;
+      bar_item_cancel_drag(bar_item);
     } else {
+      env_vars_destroy(&env_vars);
       return;
     }
   }
 
-  if (bar_item->click_script && strlen(bar_item->click_script) > 0)
-    fork_exec(bar_item->click_script, &bar_item->signal_args.env_vars);
+  if (bar_item->click_script && strlen(bar_item->click_script) > 0) {
+    for (int i = 0; i < bar_item->signal_args.env_vars.count; i++) {
+      env_vars_set(&env_vars,
+                   string_copy(bar_item->signal_args.env_vars.vars[i]->key),
+                   string_copy(bar_item->signal_args.env_vars.vars[i]->value));
+    }
+
+    fork_exec(bar_item->click_script, &env_vars);
+  }
   if (bar_item->update_mask & UPDATE_MOUSE_CLICKED)
-    bar_item_update(bar_item, COMMAND_SUBSCRIBE_MOUSE_CLICKED, true, NULL);
+    bar_item_update(bar_item,
+                    COMMAND_SUBSCRIBE_MOUSE_CLICKED,
+                    true,
+                    &env_vars                       );
+
+  env_vars_destroy(&env_vars);
+}
+
+void bar_item_on_scroll(struct bar_item* bar_item, int scroll_delta) {
+  if (!bar_item) return;
+
+  struct env_vars env_vars;
+  env_vars_init(&env_vars);
+  char info_str[256];
+  snprintf(info_str, 256, "{\n"
+                          "\t\"delta\": %d\n"
+                          "}\n",
+                          scroll_delta       );
+
+  env_vars_set(&env_vars, string_copy("INFO"), string_copy(info_str));
+
+  char delta_ver_str[32];
+  snprintf(delta_ver_str, 32, "%d", scroll_delta);
+  env_vars_set(&env_vars,
+               string_copy("SCROLL_DELTA"),
+               string_copy(delta_ver_str));
+
+  if (bar_item->update_mask & UPDATE_MOUSE_SCROLLED)
+    bar_item_update(bar_item,
+                    COMMAND_SUBSCRIBE_MOUSE_SCROLLED,
+                    true,
+                    &env_vars                        );
+
+
+  env_vars_destroy(&env_vars);
 }
 
 void bar_item_mouse_entered(struct bar_item* bar_item) {
-  bar_item->mouse_over = true;
-  if (bar_item->update_mask & UPDATE_MOUSE_ENTERED)
+  if (bar_item->update_mask & UPDATE_MOUSE_ENTERED && !bar_item->mouse_over) {
     bar_item_update(bar_item, COMMAND_SUBSCRIBE_MOUSE_ENTERED, true, NULL);
+  }
+  bar_item->mouse_over = true;
 }
 
 void bar_item_mouse_exited(struct bar_item* bar_item) {
-  if (bar_item->mouse_over && (bar_item->update_mask & UPDATE_MOUSE_EXITED))
+  if (bar_item->update_mask & UPDATE_MOUSE_EXITED && bar_item->mouse_over) {
     bar_item_update(bar_item, COMMAND_SUBSCRIBE_MOUSE_EXITED, true, NULL); 
-
+  }
   bar_item->mouse_over = false;
 }
 
@@ -243,7 +317,7 @@ static void bar_item_set_script(struct bar_item* bar_item, char* script) {
     free(bar_item->script);
 
   char* path = resolve_path(script);
-  bar_item->script = path;
+  if (path) bar_item->script = path;
 }
 
 static void bar_item_set_click_script(struct bar_item* bar_item, char* script) {
@@ -258,7 +332,7 @@ static void bar_item_set_click_script(struct bar_item* bar_item, char* script) {
     free(bar_item->click_script);
 
   char* path = resolve_path(script);
-  bar_item->click_script = path;
+  if (path) bar_item->click_script = path;
 }
 
 static bool bar_item_set_yoffset(struct bar_item* bar_item, int offset) {
@@ -340,6 +414,7 @@ bool bar_item_set_type(struct bar_item* bar_item, char* type) {
 
     bar_item->update_mask |= UPDATE_SPACE_CHANGE;
     bar_item->updates = false;
+    bar_item->updates_only_when_shown = false;
     env_vars_set(&bar_item->signal_args.env_vars,
                  string_copy("SELECTED"),
                  string_copy("false")            );
@@ -395,6 +470,14 @@ bool bar_item_set_position(struct bar_item* bar_item, char* position) {
   return true;
 }
 
+bool bar_item_set_media_cover(struct bar_item* bar_item, struct image* image) {
+  bool linked = (bar_item->background.image.link == image
+                 ||bar_item->icon.background.image.link == image
+                 ||bar_item->label.background.image.link == image);
+  if (linked) bar_item_needs_update(bar_item);
+  return linked && bar_item_is_shown(bar_item);
+}
+
 static uint32_t bar_item_get_content_length(struct bar_item* bar_item) {
   int length = text_get_length(&bar_item->icon, false)
          + text_get_length(&bar_item->label, false)
@@ -407,9 +490,10 @@ static uint32_t bar_item_get_content_length(struct bar_item* bar_item) {
 
 uint32_t bar_item_get_length(struct bar_item* bar_item, bool ignore_override) {
   uint32_t content_length = bar_item_get_content_length(bar_item);
-  if (bar_item->background.enabled && bar_item->background.image.enabled
-      && bar_item->background.image.bounds.size.width > content_length  ) {
-    content_length = bar_item->background.image.bounds.size.width;
+  if (bar_item->background.enabled && bar_item->background.image.enabled) {
+    CGSize image_size = image_get_size(&bar_item->background.image);
+
+    if (image_size.width > content_length) content_length = image_size.width;
   }
 
   if (bar_item->has_const_width && (!ignore_override
@@ -432,16 +516,14 @@ uint32_t bar_item_get_height(struct bar_item* bar_item) {
   uint32_t background_height = 0;
   if (bar_item->background.enabled) {
     uint32_t image_height = bar_item->background.image.enabled
-                            ? bar_item->background.image.bounds.size.height
+                            ? image_get_size(&bar_item->background.image).height
                             : 0;
 
     background_height = max(image_height,
                             bar_item->background.bounds.size.height);
   }
 
-  uint32_t height = max(item_height, background_height);
-
-  return height;
+  return max(item_height, background_height);
 }
 
 struct window* bar_item_get_window(struct bar_item* bar_item, uint32_t adid) {
@@ -615,6 +697,15 @@ void bar_item_clip_bar(struct bar_item* bar_item, int offset, struct bar* bar) {
   background_clip_bar(&bar_item->label.background, offset, bar);
 }
 
+void* draw_item_proc(void* context) {
+  struct { struct window* window; struct bar_item bar_item; }* info = context;
+  CGContextClearRect(info->window->context, info->window->frame);
+  bar_item_draw(&info->bar_item, info->window->context);
+  CGContextFlush(info->window->context);
+  free(context);
+  return NULL;
+}
+
 void bar_item_draw(struct bar_item* bar_item, CGContextRef context) {
   background_draw(&bar_item->background, context);
   if (bar_item->type == BAR_COMPONENT_GROUP) return;
@@ -622,12 +713,9 @@ void bar_item_draw(struct bar_item* bar_item, CGContextRef context) {
   text_draw(&bar_item->icon, context);
   text_draw(&bar_item->label, context);
 
-  if (bar_item->has_alias)
-    alias_draw(&bar_item->alias, context);
-  if (bar_item->has_graph)
-    graph_draw(&bar_item->graph, context);
-  if (bar_item->has_slider)
-    slider_draw(&bar_item->slider, context);
+  if (bar_item->has_alias) alias_draw(&bar_item->alias, context);
+  if (bar_item->has_graph) graph_draw(&bar_item->graph, context);
+  if (bar_item->has_slider) slider_draw(&bar_item->slider, context);
 }
 
 void bar_item_change_space(struct bar_item* bar_item, uint64_t dsid, uint32_t adid) {
@@ -645,12 +733,13 @@ static void bar_item_clear_pointers(struct bar_item* bar_item) {
   bar_item->signal_args.env_vars.vars = NULL;
   bar_item->signal_args.env_vars.count = 0;
   bar_item->windows = NULL;
-  bar_item->popup.host = bar_item;
   bar_item->num_windows = 0;
   text_clear_pointers(&bar_item->icon);
   text_clear_pointers(&bar_item->label);
   background_clear_pointers(&bar_item->background);
   slider_clear_pointers(&bar_item->slider);
+  popup_clear_pointers(&bar_item->popup);
+  bar_item->popup.host = bar_item;
 }
 
 void bar_item_inherit_from_item(struct bar_item* bar_item, struct bar_item* ancestor) {
@@ -663,22 +752,15 @@ void bar_item_inherit_from_item(struct bar_item* bar_item, struct bar_item* ance
   char* click_script = bar_item->click_script;
 
   memcpy(bar_item, ancestor, sizeof(struct bar_item));
-
   bar_item_clear_pointers(bar_item);
+
   bar_item->name = name;
   bar_item->script = script;
   bar_item->click_script = click_script;
 
-  text_set_font(&bar_item->icon, string_copy(ancestor->icon.font_name), true);
-  text_set_font(&bar_item->label, string_copy(ancestor->label.font_name),true);
-  text_set_string(&bar_item->icon, string_copy(ancestor->icon.string), true);
-  text_set_string(&bar_item->label, string_copy(ancestor->label.string), true);
-
-  text_set_font(&bar_item->slider.knob,
-                string_copy(ancestor->slider.knob.font_name), true);
-
-  text_set_string(&bar_item->slider.knob,
-                  string_copy(ancestor->slider.knob.string), true);
+  text_copy(&bar_item->icon, &ancestor->icon);
+  text_copy(&bar_item->label, &ancestor->label);
+  text_copy(&bar_item->slider.knob, &ancestor->slider.knob);
 
   if (ancestor->script)
     bar_item_set_script(bar_item, string_copy(ancestor->script));
@@ -710,7 +792,7 @@ void bar_item_inherit_from_item(struct bar_item* bar_item, struct bar_item* ance
   }
 }
 
-void bar_item_destroy(struct bar_item* bar_item) {
+void bar_item_destroy(struct bar_item* bar_item, bool free_memory) {
   if (bar_item->name) free(bar_item->name);
   if (bar_item->script) free(bar_item->script);
   if (bar_item->click_script) free(bar_item->click_script);
@@ -736,7 +818,7 @@ void bar_item_destroy(struct bar_item* bar_item) {
   }
   if (bar_item->windows) free(bar_item->windows);
 
-  free(bar_item);
+  if (free_memory) free(bar_item);
 }
 
 void bar_item_serialize(struct bar_item* bar_item, FILE* rsp) {
@@ -953,14 +1035,18 @@ void bar_item_parse_set_message(struct bar_item* bar_item, char* message, FILE*
     }
   }
   else if (token_equals(property, PROPERTY_ICON)) {
-    needs_refresh = text_set_string(&bar_item->icon,
-                                    token_to_string(get_token(&message)),
-                                    false                                );
+    struct token dummy = { PROPERTY_STRING, strlen(PROPERTY_STRING)};
+    needs_refresh = text_parse_sub_domain(&bar_item->icon,
+                                          rsp,
+                                          dummy,
+                                          message         );
 
   } else if (token_equals(property, PROPERTY_LABEL)) {
-    needs_refresh = text_set_string(&bar_item->label,
-                                    token_to_string(get_token(&message)),
-                                    false                                );
+    struct token dummy = { PROPERTY_STRING, strlen(PROPERTY_STRING)};
+    needs_refresh = text_parse_sub_domain(&bar_item->label,
+                                          rsp,
+                                          dummy,
+                                          message          );
 
   } else if (token_equals(property, PROPERTY_UPDATES)) {
     struct token token = get_token(&message);
@@ -976,6 +1062,9 @@ void bar_item_parse_set_message(struct bar_item* bar_item, char* message, FILE*
     needs_refresh = bar_item_set_drawing(bar_item,
                                          evaluate_boolean_state(get_token(&message),
                                                                 bar_item->drawing   ));
+  } else if (token_equals(property, PROPERTY_SCROLL_TEXTS)) {
+    bar_item->scroll_texts = evaluate_boolean_state(get_token(&message),
+                                                    bar_item->scroll_texts);
   } else if (token_equals(property, PROPERTY_WIDTH)) {
     struct token token = get_token(&message);
     if (token_equals(token, ARGUMENT_DYNAMIC)) {
@@ -992,7 +1081,7 @@ void bar_item_parse_set_message(struct bar_item* bar_item, char* message, FILE*
                       (bool (*)(void*, int))&bar_item_set_width,
                       bar_item->custom_width,
                       -1,
-                      1,
+                      0,
                       INTERP_FUNCTION_LINEAR               );
       animator_add(&g_bar_manager.animator, animation);
     }
@@ -1039,7 +1128,8 @@ void bar_item_parse_set_message(struct bar_item* bar_item, char* message, FILE*
       bar_item->align = position.text[0];
       needs_refresh = true;
     }
-  } else if (token_equals(property, PROPERTY_ASSOCIATED_SPACE)) {
+  } else if (token_equals(property, PROPERTY_ASSOCIATED_SPACE)
+             || token_equals(property, PROPERTY_SPACE)        ) {
     struct token token = get_token(&message);
     uint32_t prev = bar_item->associated_space;
     bar_item->associated_space = 0;
@@ -1055,7 +1145,8 @@ void bar_item_parse_set_message(struct bar_item* bar_item, char* message, FILE*
       free(list);
     }
     needs_refresh = (prev != bar_item->associated_space);
-  } else if (token_equals(property, PROPERTY_ASSOCIATED_DISPLAY)) {
+  } else if (token_equals(property, PROPERTY_ASSOCIATED_DISPLAY)
+             || token_equals(property, PROPERTY_DISPLAY)        ) {
     struct token token = get_token(&message);
     uint32_t prev = bar_item->associated_display;
     bar_item->associated_display = 0;
@@ -1147,6 +1238,14 @@ void bar_item_parse_subscribe_message(struct bar_item* bar_item, char* message,
       begin_receiving_brightness_events();
     }
 
+    if (event_flag & UPDATE_MEDIA_CHANGE) {
+      begin_receiving_media_events();
+    }
+
+    if (event_flag & UPDATE_SPACE_WINDOWS_CHANGE) {
+      begin_receiving_space_window_events();
+    }
+
     bar_item->update_mask |= event_flag;
 
     if (!event_flag) {
