@@ -1,15 +1,21 @@
 #include "bar_manager.h"
+#include "bar.h"
 #include "bar_item.h"
 #include "event.h"
-#include "event_loop.h"
 #include "misc/env_vars.h"
 #include "misc/helpers.h"
+#include "wifi.h"
+#include "volume.h"
+#include "power.h"
+#include "mouse.h"
+#include "media.h"
+#include "app_windows.h"
 
-extern struct event_loop g_event_loop;
+extern void forced_front_app_event();
 
 static CLOCK_CALLBACK(clock_handler) {
-  struct event *event = event_create(&g_event_loop, SHELL_REFRESH, NULL);
-  event_loop_post(&g_event_loop, event);
+  struct event event = { NULL, SHELL_REFRESH };
+  event_post(&event);
 }
 
 void bar_manager_init(struct bar_manager* bar_manager) {
@@ -19,8 +25,9 @@ void bar_manager_init(struct bar_manager* bar_manager) {
   bar_manager->bar_needs_update = false;
   bar_manager->bars = NULL;
   bar_manager->bar_count = 0;
+  bar_manager->bar_items = NULL;
   bar_manager->bar_item_count = 0;
-  bar_manager->display = DISPLAY_ALL;
+  bar_manager->displays = DISPLAY_ALL_PATTERN;
   bar_manager->position = POSITION_TOP;
   bar_manager->shadow = false;
   bar_manager->blur_radius = 0;
@@ -29,20 +36,22 @@ void bar_manager_init(struct bar_manager* bar_manager) {
   bar_manager->sleeps = false;
   bar_manager->window_level = kCGBackstopMenuLevel;
   bar_manager->topmost = false;
-  bar_manager->sticky = false;
-  bar_manager->picky_redraw = false;
   bar_manager->notch_width = 200;
   bar_manager->notch_offset = 0;
-  bar_manager->active_adid = display_arrangement(display_active_display_id());
+  bar_manager->active_adid = display_active_display_adid();
   bar_manager->might_need_clipping = false;
 
+  bar_manager->sticky = true;
+
+  image_init(&bar_manager->current_artwork);
   background_init(&bar_manager->background);
   bar_manager->background.bounds.size.height = 25;
   bar_manager->background.overrides_height = true;
   bar_manager->background.padding_left = 20;
   bar_manager->background.padding_right = 20;
-  bar_manager->background.border_color = rgba_color_from_hex(0xffff0000);
-  bar_manager->background.color = rgba_color_from_hex(0x44000000);
+
+  color_set_hex(&bar_manager->background.border_color, 0xffff0000);
+  color_set_hex(&bar_manager->background.color, 0x44000000);
 
   bar_item_init(&bar_manager->default_item, NULL);
   bar_item_set_name(&bar_manager->default_item, string_copy("defaults"));
@@ -165,7 +174,7 @@ void bar_manager_remove_item(struct bar_manager* bar_manager, struct bar_item* b
            sizeof(struct bar_item*)*bar_manager->bar_item_count);
   }
 
-  bar_item_destroy(bar_item);
+  bar_item_destroy(bar_item, true);
 }
 
 bool bar_manager_set_margin(struct bar_manager* bar_manager, int margin) {
@@ -203,9 +212,9 @@ bool bar_manager_set_position(struct bar_manager* bar_manager, char pos) {
   return true;
 }
 
-bool bar_manager_set_display(struct bar_manager* bar_manager, char display) {
-  if (bar_manager->display == display) return false;
-  bar_manager->display = display;
+bool bar_manager_set_displays(struct bar_manager* bar_manager, uint32_t displays) {
+  if (bar_manager->displays == displays) return false;
+  bar_manager->displays = displays;
 
   bar_manager_reset(bar_manager);
   return true;
@@ -264,9 +273,17 @@ bool bar_manager_set_hidden(struct bar_manager *bar_manager, uint32_t adid, bool
   return true;
 }
 
-bool bar_manager_set_topmost(struct bar_manager *bar_manager, bool topmost) {
-  if (topmost) bar_manager->window_level = kCGScreenSaverWindowLevel;
-  else bar_manager->window_level = kCGBackstopMenuLevel;
+bool bar_manager_set_topmost(struct bar_manager *bar_manager, char level, bool topmost) {
+  if (topmost) {
+    if (level == TOPMOST_LEVEL_WINDOW) {
+      bar_manager->window_level = kCGFloatingWindowLevel;
+    } else if (level == TOPMOST_LEVEL_ALL) {
+      bar_manager->window_level = kCGStatusWindowLevel;
+    }
+  } else {
+    bar_manager->window_level = kCGBackstopMenuLevel;
+  }
+
   bar_manager_reset(bar_manager);
   bar_manager->topmost = topmost;
   return true;
@@ -292,7 +309,9 @@ uint32_t bar_manager_length_for_bar_side(struct bar_manager* bar_manager, struct
   uint32_t total_length = 0;
   for (int i = 0; i < bar_manager->bar_item_count; i++) {
     struct bar_item* bar_item = bar_manager->bar_items[i];
-    if (bar_item->position == side && bar_draws_item(bar, bar_item)) {
+    if (bar_item->position == side
+        && bar_item->type != BAR_COMPONENT_GROUP
+        && bar_draws_item(bar, bar_item)        ) {
       int item_length = (bar_manager->position == POSITION_LEFT
                          || bar_manager->position == POSITION_RIGHT)
                         ? bar_item_get_height(bar_item)
@@ -325,26 +344,7 @@ bool bar_manager_bar_needs_redraw(struct bar_manager* bar_manager, struct bar* b
 
     if (disabled_item_drawn_on_bar) return true;
 
-    if (bar_item->ignore_association
-        || (bar_item->type == BAR_COMPONENT_SPACE))
-      continue;
-
-    bool drawn_on_non_associated_space = bar_item->associated_space > 0
-                                         && !(bar_item->associated_space
-                                              & (1 << bar->sid))
-                                         && ((bar_item->associated_bar << 1)
-                                             & bar_mask);
-
-    if (drawn_on_non_associated_space) return true;
-
-    bool not_drawn_on_associated_space = draws_item
-                                         && bar_item->associated_space > 0
-                                         && (bar_item->associated_space
-                                             & (1 << bar->sid))
-                                         && !((bar_item->associated_bar << 1)
-                                              & bar_mask);
-
-    if (not_drawn_on_associated_space) return true;
+    if (bar_item->ignore_association) continue;
 
     bool not_drawn_on_associated_display =
       (draws_item
@@ -370,6 +370,25 @@ bool bar_manager_bar_needs_redraw(struct bar_manager* bar_manager, struct bar* b
 
     if (drawn_on_non_associated_display) return true;
 
+    if (bar_item->type == BAR_COMPONENT_SPACE) continue;
+
+    bool drawn_on_non_associated_space = bar_item->associated_space > 0
+                                         && !(bar_item->associated_space
+                                              & (1 << bar->sid))
+                                         && ((bar_item->associated_bar << 1)
+                                             & bar_mask);
+
+    if (drawn_on_non_associated_space) return true;
+
+    bool not_drawn_on_associated_space = draws_item
+                                         && bar_item->associated_space > 0
+                                         && (bar_item->associated_space
+                                             & (1 << bar->sid))
+                                         && !((bar_item->associated_bar << 1)
+                                              & bar_mask);
+
+    if (not_drawn_on_associated_space) return true;
+
   }
   return false;
 }
@@ -387,7 +406,7 @@ void bar_manager_reset_bar_association(struct bar_manager* bar_manager) {
     bar_item_reset_associated_bar(bar_manager->bar_items[i]);
 }
 
-void bar_manager_refresh(struct bar_manager* bar_manager, bool forced) {
+void bar_manager_refresh(struct bar_manager* bar_manager, bool forced, bool threaded) {
   if (bar_manager->frozen) return;
   if (forced) {
     bar_manager_reset_bar_association(bar_manager);
@@ -396,11 +415,13 @@ void bar_manager_refresh(struct bar_manager* bar_manager, bool forced) {
     }
   }
 
+  if (forced || bar_manager->bar_needs_resize) bar_manager_resize(bar_manager);
+
   for (int i = 0; i < bar_manager->bar_count; ++i) {
     if (forced
         || bar_manager_bar_needs_redraw(bar_manager, bar_manager->bars[i])) { 
       bar_calculate_bounds(bar_manager->bars[i]);
-      bar_draw(bar_manager->bars[i]);
+      bar_draw(bar_manager->bars[i], false, threaded);
       if (bar_manager->needs_ordering) {
         bar_order_item_windows(bar_manager->bars[i]);
       }
@@ -408,6 +429,7 @@ void bar_manager_refresh(struct bar_manager* bar_manager, bool forced) {
   }
 
   bar_manager_clear_needs_update(bar_manager);
+  if (threaded) join_render_threads();
 }
 
 void bar_manager_resize(struct bar_manager* bar_manager) {
@@ -437,7 +459,7 @@ void bar_manager_update_alias_components(struct bar_manager* bar_manager, bool f
       continue;
     }
 
-    bar_item_update(bar_manager->bar_items[i], NULL, forced, NULL);
+    alias_update(&bar_manager->bar_items[i]->alias, forced);
   }
 }
 
@@ -487,19 +509,31 @@ void bar_manager_update_space_components(struct bar_manager* bar_manager, bool f
   }
 }
 
-void bar_manager_animator_refresh(struct bar_manager* bar_manager) {
+void bar_manager_animator_refresh(struct bar_manager* bar_manager, uint64_t time) {
   bar_manager_freeze(bar_manager);
-  if (animator_update(&bar_manager->animator)) {
+  if (animator_update(&bar_manager->animator, time)) {
     bar_manager_unfreeze(bar_manager);
 
     if (bar_manager->bar_needs_resize) bar_manager_resize(bar_manager);
-    bar_manager_refresh(bar_manager, false);
+    bar_manager_refresh(bar_manager, false, true);
   }
   bar_manager_unfreeze(bar_manager);
 }
 
 void bar_manager_update(struct bar_manager* bar_manager, bool forced) {
   if ((bar_manager->frozen && !forced) || bar_manager->sleeps) return;
+
+  if (forced) {
+    bar_manager_handle_space_change(bar_manager, true);
+    forced_network_event();
+    forced_volume_event();
+    forced_brightness_event();
+    forced_power_event();
+    forced_front_app_event();
+    forced_media_change_event();
+    forced_space_windows_event();
+  }
+
   bool needs_refresh = false;
   for (int i = 0; i < bar_manager->bar_item_count; i++) {
     struct bar_item* bar_item = bar_manager->bar_items[i];
@@ -507,13 +541,13 @@ void bar_manager_update(struct bar_manager* bar_manager, bool forced) {
 
     if (bar_item->has_alias
         && bar_item_is_shown(bar_item)
-        && alias_update(&bar_item->alias)) {
+        && alias_update(&bar_item->alias, false)) {
       bar_item_needs_update(bar_item);
       needs_refresh = true;
     }
   }
 
-  if (needs_refresh) bar_manager_refresh(bar_manager, false);
+  if (needs_refresh || forced) bar_manager_refresh(bar_manager, forced, false);
 }
 
 void bar_manager_reset(struct bar_manager* bar_manager) {
@@ -533,7 +567,7 @@ void bar_manager_reset(struct bar_manager* bar_manager) {
 }
 
 void bar_manager_begin(struct bar_manager* bar_manager) {
-  if (bar_manager->display == DISPLAY_MAIN) {
+  if (bar_manager->displays == DISPLAY_MAIN_PATTERN) {
     uint32_t did = display_main_display_id();
     bar_manager->bar_count = 1;
     bar_manager->bars = (struct bar **) realloc(
@@ -542,19 +576,33 @@ void bar_manager_begin(struct bar_manager* bar_manager) {
 
     memset(bar_manager->bars, 0, sizeof(struct bar*) * bar_manager->bar_count);
     bar_manager->bars[0] = bar_create(did);
-    bar_manager->bars[0]->adid = 1;
+    bar_manager->bars[0]->adid = display_arrangement(did);
   } 
   else {
-    bar_manager->bar_count = display_active_display_count();
+    uint32_t display_count = display_active_display_count();
+    uint32_t bar_count = 0;
+    for (uint32_t index = 1; index <= display_count; index++) {
+      if (!(bar_manager->displays & 1 << (index - 1))) continue;
+      bar_count++;
+    }
+
+    bar_manager->bar_count = bar_count;
     bar_manager->bars = (struct bar **) realloc(
                                 bar_manager->bars,
                                 sizeof(struct bar *) * bar_manager->bar_count);
 
     memset(bar_manager->bars, 0, sizeof(struct bar*) * bar_manager->bar_count);
-    for (uint32_t index=1; index <= bar_manager->bar_count; index++) {
+
+    uint32_t bar_index = 0;
+    for (uint32_t index = 1; index <= display_count; index++) {
+      if (!(bar_manager->displays & 1 << (index - 1))) continue;
       uint32_t did = display_arrangement_display_id(index);
-      bar_manager->bars[index - 1] = bar_create(did);
-      bar_manager->bars[index - 1]->adid = index;
+      bar_manager->bars[bar_index] = bar_create(did);
+      bar_manager->bars[bar_index]->adid = index;
+      if (bar_manager->any_bar_hidden)
+        bar_set_hidden(bar_manager->bars[bar_index], true);
+
+      bar_index++;
     }
   }
 
@@ -563,43 +611,43 @@ void bar_manager_begin(struct bar_manager* bar_manager) {
     bar_manager->active_displays |= 1 << bar_manager->bars[i]->adid;
   }
 
+  bar_manager->active_adid = display_active_display_adid();
   bar_manager->needs_ordering = true;
 }
 
-struct bar_item* bar_manager_get_item_by_point(struct bar_manager* bar_manager, CGPoint point, uint32_t adid) {
-  if (adid < 1) return NULL;
-
+struct bar_item* bar_manager_get_item_by_point(struct bar_manager* bar_manager, CGPoint point, struct window** window_out) {
   for (int i = 0; i < bar_manager->bar_item_count; i++) {
     struct bar_item* bar_item = bar_manager->bar_items[i];
-    if (!bar_item->drawing || bar_item->num_windows < adid
-        || !bar_item->windows[adid - 1]                   ) {
-      continue;
-    }
+    if (!bar_item->drawing) continue;
 
-    struct window* window = bar_item_get_window(bar_item, adid);
-    CGRect frame = window->frame;
-    frame.origin = window->origin;
-    if (cgrect_contains_point(&frame, &point)) {
-      return bar_item;
+    for (int adid = 1; adid <= bar_item->num_windows; adid++) {
+      struct window* window = bar_item_get_window(bar_item, adid);
+      if (!window) continue;
+
+      CGRect frame = window->frame;
+      frame.origin = window->origin;
+      if (cgrect_contains_point(&frame, &point)) {
+        if (window_out) *window_out = window;
+        return bar_item;
+      }
     }
   }
   return NULL;
 }
 
-struct bar_item* bar_manager_get_item_by_wid(struct bar_manager* bar_manager, uint32_t wid, uint32_t adid) {
-  if (adid < 1) return NULL;
-
+struct bar_item* bar_manager_get_item_by_wid(struct bar_manager* bar_manager, uint32_t wid, struct window** window_out) {
   for (int i = 0; i < bar_manager->bar_item_count; i++) {
     struct bar_item* bar_item = bar_manager->bar_items[i];
-    if (!bar_item->drawing || bar_item->num_windows < adid
-        || !bar_item->windows[adid - 1]) {
-      continue;
-    }
+    if (!bar_item->drawing) continue;
 
-    struct window* window = bar_item_get_window(bar_item, adid);
+    for (int adid = 1; adid <= bar_item->num_windows; adid++) {
+      struct window* window = bar_item_get_window(bar_item, adid);
+      if (!window) continue;
 
-    if (window->id == wid) {
-      return bar_item;
+      if (window->id == wid) {
+        if (window_out) *window_out = window;
+        return bar_item;
+      }
     }
   }
   return NULL;
@@ -689,15 +737,55 @@ void bar_manager_custom_events_trigger(struct bar_manager* bar_manager, char* na
   }
 }
 
+void bar_manager_display_resized(struct bar_manager* bar_manager, uint32_t did) {
+  for (int i = 0; i < bar_manager->bar_count; i++) {
+    if (bar_manager->bars[i]->did == did) {
+      bar_resize(bar_manager->bars[i]);
+    }
+  }
+  bar_manager_refresh(bar_manager, false, false);
+}
+
+void bar_manager_display_moved(struct bar_manager* bar_manager, uint32_t did) {
+  for (int i = 0; i < bar_manager->bar_count; i++) {
+    struct bar* bar = bar_manager->bars[i];
+    if (bar->did == did) {
+      bar_resize(bar);
+      bar->adid = display_arrangement(bar->did);
+      bar->dsid = display_space_id(bar->did);
+      bar->sid = mission_control_index(bar->dsid);
+    }
+  }
+  bar_manager->needs_ordering = true;
+  bar_manager_refresh(bar_manager, true, false);
+  bar_manager_handle_display_change(bar_manager);
+}
+
+void bar_manager_display_removed(struct bar_manager* bar_manager, uint32_t did) {
+  bar_manager_display_changed(bar_manager);
+}
+
+void bar_manager_display_added(struct bar_manager* bar_manager, uint32_t did) {
+  bar_manager_display_changed(bar_manager);
+}
+
 void bar_manager_display_changed(struct bar_manager* bar_manager) {
-  bar_manager->active_adid = display_arrangement(display_active_display_id());
+  bar_manager->active_adid = display_active_display_adid();
 
   bar_manager_freeze(bar_manager);
   bar_manager_reset(bar_manager);
   bar_manager_unfreeze(bar_manager);
-  bar_manager_refresh(bar_manager, true);
+  bar_manager_refresh(bar_manager, true, false);
 
+  bar_manager_handle_display_change(bar_manager);
   bar_manager_handle_space_change(bar_manager, true);
+  animator_renew_display_link(&bar_manager->animator);
+}
+
+void bar_manager_cancel_drag(struct bar_manager* bar_manager) {
+  for (uint32_t i = 0; i < bar_manager->bar_item_count; i++) {
+    bar_item_cancel_drag(bar_manager->bar_items[i]);
+  }
 }
 
 void bar_manager_handle_mouse_entered_global(struct bar_manager* bar_manager) {
@@ -710,10 +798,41 @@ void bar_manager_handle_mouse_exited_global(struct bar_manager* bar_manager) {
   bar_manager_custom_events_trigger(bar_manager,
                                     COMMAND_SUBSCRIBE_MOUSE_EXITED_GLOBAL,
                                     NULL                                  );
+
+  bar_manager_handle_mouse_exited(bar_manager, NULL);
+}
+
+void bar_manager_handle_mouse_scrolled_global(struct bar_manager* bar_manager, int scroll_delta, uint32_t adid) {
+  struct env_vars env_vars;
+  env_vars_init(&env_vars);
+  char delta_ver_str[32];
+  snprintf(delta_ver_str, 32, "%d", scroll_delta);
+  env_vars_set(&env_vars,
+               string_copy("SCROLL_DELTA"),
+               string_copy(delta_ver_str));
+
+  char info_str[256];
+  snprintf(info_str, 256, "{\n"
+                          "\t\"delta\": %d\n"
+                          "}\n",
+                          scroll_delta       );
+
+  env_vars_set(&env_vars, string_copy("INFO"), string_copy(info_str));
+
+  char adid_str[32];
+  snprintf(adid_str, 32, "%u", adid);
+  env_vars_set(&env_vars,
+               string_copy("DID"),
+               string_copy(adid_str));
+
+  bar_manager_custom_events_trigger(bar_manager,
+                                    COMMAND_SUBSCRIBE_MOUSE_SCROLLED_GLOBAL,
+                                    &env_vars                               );
+  env_vars_destroy(&env_vars);
 }
 
 void bar_manager_handle_mouse_entered(struct bar_manager* bar_manager, struct bar_item* bar_item) {
-  if (!bar_item || bar_item->mouse_over) return;
+  if (!bar_item) return;
   bar_item_mouse_entered(bar_item);
 }
 
@@ -738,6 +857,16 @@ void bar_manager_handle_volume_change(struct bar_manager* bar_manager, float vol
   env_vars_destroy(&env_vars);
 }
 
+void bar_manager_handle_wifi_change(struct bar_manager* bar_manager, char* ssid) {
+  struct env_vars env_vars;
+  env_vars_init(&env_vars);
+  env_vars_set(&env_vars, string_copy("INFO"), string_copy(ssid));
+  bar_manager_custom_events_trigger(bar_manager,
+                                    COMMAND_SUBSCRIBE_WIFI_CHANGE,
+                                    &env_vars                     );
+  env_vars_destroy(&env_vars);
+}
+
 void bar_manager_handle_brightness_change(struct bar_manager* bar_manager, float brightness) {
   struct env_vars env_vars;
   env_vars_init(&env_vars);
@@ -760,6 +889,29 @@ void bar_manager_handle_power_source_change(struct bar_manager* bar_manager, cha
   env_vars_destroy(&env_vars);
 }
 
+void bar_manager_handle_media_change(struct bar_manager* bar_manager, char* info) {
+  struct env_vars env_vars;
+  env_vars_init(&env_vars);
+  env_vars_set(&env_vars, string_copy("INFO"), string_copy(info));
+  bar_manager_custom_events_trigger(bar_manager,
+                                    COMMAND_SUBSCRIBE_MEDIA_CHANGE,
+                                    &env_vars                      );
+  env_vars_destroy(&env_vars);
+}
+
+void bar_manager_handle_media_cover_change(struct bar_manager* bar_manager, CGImageRef image) {
+  bool needs_refresh = false;
+  if (image_set_image(&bar_manager->current_artwork, image, CGRectNull, false)){
+    for (int i = 0; i < bar_manager->bar_item_count; i++) {
+      struct bar_item* bar_item = bar_manager->bar_items[i];
+      needs_refresh |= bar_item_set_media_cover(bar_item,
+                                                &bar_manager->current_artwork);
+    }
+  }
+
+  if (needs_refresh) bar_manager_refresh(bar_manager, false, false);
+}
+
 void bar_manager_handle_front_app_switch(struct bar_manager* bar_manager, char* info) {
   struct env_vars env_vars;
   env_vars_init(&env_vars);
@@ -770,6 +922,16 @@ void bar_manager_handle_front_app_switch(struct bar_manager* bar_manager, char*
   env_vars_destroy(&env_vars);
 }
 
+void bar_manager_handle_space_windows_change(struct bar_manager* bar_manager, char* info) {
+  struct env_vars env_vars;
+  env_vars_init(&env_vars);
+  if (info) env_vars_set(&env_vars, string_copy("INFO"), string_copy(info));
+  bar_manager_custom_events_trigger(bar_manager,
+                                    COMMAND_SUBSCRIBE_SPACE_WINDOWS_CHANGE,
+                                    &env_vars                              );
+  env_vars_destroy(&env_vars);
+}
+
 void bar_manager_handle_space_change(struct bar_manager* bar_manager, bool forced) {
   struct env_vars env_vars;
   env_vars_init(&env_vars);
@@ -789,6 +951,7 @@ void bar_manager_handle_space_change(struct bar_manager* bar_manager, bool force
 
     bar_manager->needs_ordering |= !was_shown && bar_manager->bars[i]->shown;
     force_refresh |= !was_shown && bar_manager->bars[i]->shown;
+    force_refresh |= !bar_manager->bars[i]->shown && was_shown;
 
     if (bar_manager->bars[i]->dsid != dsid) {
       bar_manager->bars[i]->dsid = dsid;
@@ -817,12 +980,12 @@ void bar_manager_handle_space_change(struct bar_manager* bar_manager, bool force
 
 
   bar_manager_unfreeze(bar_manager);
-  bar_manager_refresh(bar_manager, force_refresh);
+  bar_manager_refresh(bar_manager, force_refresh, false);
   env_vars_destroy(&env_vars);
 }
 
 void bar_manager_handle_display_change(struct bar_manager* bar_manager) {
-  bar_manager->active_adid = display_arrangement(display_active_display_id());
+  bar_manager->active_adid = display_active_display_adid();
   struct env_vars env_vars;
   env_vars_init(&env_vars);
   char adid_str[3];
@@ -833,7 +996,7 @@ void bar_manager_handle_display_change(struct bar_manager* bar_manager) {
                                     COMMAND_SUBSCRIBE_DISPLAY_CHANGE,
                                     &env_vars                        );
 
-  bar_manager_refresh(bar_manager, false);
+  bar_manager_refresh(bar_manager, false, false);
   env_vars_destroy(&env_vars);
 }
 
@@ -841,17 +1004,17 @@ void bar_manager_handle_system_will_sleep(struct bar_manager* bar_manager) {
   bar_manager_custom_events_trigger(bar_manager,
                                     COMMAND_SUBSCRIBE_SYSTEM_WILL_SLEEP,
                                     NULL                                );
+  animator_destroy_display_link(&bar_manager->animator);
   bar_manager->sleeps = true;
 }
 
 void bar_manager_handle_system_woke(struct bar_manager* bar_manager) {
+  usleep(100000);
   bar_manager->sleeps = false;
+  bar_manager_display_changed(bar_manager);
   bar_manager_custom_events_trigger(bar_manager,
                                     COMMAND_SUBSCRIBE_SYSTEM_WOKE,
                                     NULL                          );
-
-  bar_manager_refresh(bar_manager, true);
-  bar_manager_handle_space_change(bar_manager, true);
 }
 
 void bar_manager_handle_notification(struct bar_manager* bar_manager, struct notification* notification) {
@@ -874,16 +1037,35 @@ void bar_manager_handle_notification(struct bar_manager* bar_manager, struct not
 
 void bar_manager_destroy(struct bar_manager* bar_manager) {
   for (int i = 0; i < bar_manager->bar_item_count; i++) {
-    bar_item_destroy(bar_manager->bar_items[i]);
+    struct bar_item* bar_item = bar_manager->bar_items[i];
+    if (bar_item->event_port) {
+      mach_send_message(bar_item->event_port, "k", 2, false);
+    }
   }
+
+  animator_destroy(&bar_manager->animator);
+
+  while (bar_manager->bar_item_count > 0) {
+    bar_manager_remove_item(bar_manager, bar_manager->bar_items[0]);
+  }
+
   if (bar_manager->bar_items) free(bar_manager->bar_items);
   for (int i = 0; i < bar_manager->bar_count; i++) {
     bar_destroy(bar_manager->bars[i]);
   }
+
+  bar_item_destroy(&bar_manager->default_item, false);
   custom_events_destroy(&bar_manager->custom_events);
   background_destroy(&bar_manager->background);
 
   if (bar_manager->bars) free(bar_manager->bars);
+  CFRunLoopRemoveTimer(CFRunLoopGetMain(),
+                       bar_manager->clock,
+                       kCFRunLoopCommonModes);
+
+  CFRunLoopTimerInvalidate(bar_manager->clock);
+  CFRelease(bar_manager->clock);
+  image_destroy(&bar_manager->current_artwork);
 }
 
 void bar_manager_serialize(struct bar_manager* bar_manager, FILE* rsp) {
@@ -892,6 +1074,7 @@ void bar_manager_serialize(struct bar_manager* bar_manager, FILE* rsp) {
                "%s\"position\": \"%s\",\n"
                "%s\"topmost\": \"%s\",\n"
                "%s\"sticky\": \"%s\",\n"
+               "%s\"hidden\": \"%s\",\n"
                "%s\"shadow\": \"%s\",\n"
                "%s\"font_smoothing\": \"%s\",\n"
                "%s\"blur_radius\": %u,\n"
@@ -900,6 +1083,7 @@ void bar_manager_serialize(struct bar_manager* bar_manager, FILE* rsp) {
                                               ? "bottom" : "top",
                indent, format_bool(bar_manager->topmost),
                indent, format_bool(bar_manager->sticky),
+               indent, format_bool(bar_manager->any_bar_hidden),
                indent, format_bool(bar_manager->shadow),
                indent, format_bool(bar_manager->font_smoothing),
                indent, bar_manager->blur_radius,
