@@ -1,39 +1,6 @@
 #include "group.h"
 #include "bar.h"
 
-struct group* group_create() {
-  struct group* group = malloc(sizeof(struct group));
-  memset(group, 0, sizeof(struct group));
-  return group;
-}
-
-void group_init(struct group* group) {
-  group->num_members = 0;
-  group->members = NULL;
-}
-
-bool group_is_item_member(struct group* group, struct bar_item* item) {
-  for (uint32_t i = 0; i < group->num_members; i++) {
-    if (group->members[i] == item) return true;
-  }
-  return false;
-}
-
-void group_add_member(struct group* group, struct bar_item* item) {
-  if (group_is_item_member(group, item)) return;
-  if (item->group && item->group->members && item->group->members[0] == item) {
-    for (int i = 1; i < item->group->num_members; i++) {
-      group_add_member(group, item->group->members[i]);
-    }
-  } else {
-    group->num_members++;
-    group->members = realloc(group->members,
-                             sizeof(struct bar_item*)*group->num_members);
-    group->members[group->num_members - 1] = item;
-    item->group = group;
-  }
-}
-
 static struct bar_item* group_get_first_member(struct group* group, struct bar* bar) {
   if (group->num_members == 1) return NULL;
 
@@ -74,6 +41,39 @@ static struct bar_item* group_get_last_member(struct group* group, struct bar* b
   return last_item;
 }
 
+struct group* group_create() {
+  struct group* group = malloc(sizeof(struct group));
+  memset(group, 0, sizeof(struct group));
+  return group;
+}
+
+void group_init(struct group* group) {
+  group->num_members = 0;
+  group->members = NULL;
+}
+
+bool group_is_item_member(struct group* group, struct bar_item* item) {
+  for (uint32_t i = 0; i < group->num_members; i++) {
+    if (group->members[i] == item) return true;
+  }
+  return false;
+}
+
+void group_add_member(struct group* group, struct bar_item* item) {
+  if (group_is_item_member(group, item)) return;
+  if (item->group && item->group->members && item->group->members[0] == item) {
+    for (int i = 1; i < item->group->num_members; i++) {
+      group_add_member(group, item->group->members[i]);
+    }
+  } else {
+    group->num_members++;
+    group->members = realloc(group->members,
+                             sizeof(struct bar_item*)*group->num_members);
+    group->members[group->num_members - 1] = item;
+    item->group = group;
+  }
+}
+
 uint32_t group_get_length(struct group* group, struct bar* bar) {
   int len = group->last_window->origin.x
             + group->last_window->frame.size.width
@@ -137,10 +137,6 @@ void group_calculate_bounds(struct group* group, struct bar* bar, uint32_t y) {
                               group->members[0]->background.bounds.size.height);
 }
 
-void group_draw(struct group* group, CGContextRef context) {
-  background_draw(&group->members[0]->background, context);
-}
-
 void group_serialize(struct group* group, char* indent, FILE* rsp) {
     int counter = 0;
     for (int i = 1; i < group->num_members; i++) {
