@@ -1,7 +1,12 @@
 #include "message.h"
+#include "app_windows.h"
 #include "misc/defines.h"
+#include "hotload.h"
+#include "volume.h"
+#include "media.h"
+#include "wifi.h"
+#include "power.h"
 
-extern struct event_loop g_event_loop;
 extern struct bar_manager g_bar_manager;
 
 static struct bar_item** get_bar_items_for_regex(struct token reg, FILE* rsp, uint32_t* count) {
@@ -75,9 +80,18 @@ static void handle_domain_trigger(FILE* rsp, struct token domain, char* message)
 
   if (token_equals(event, COMMAND_SUBSCRIBE_SPACE_CHANGE)) {
     bar_manager_handle_space_change(&g_bar_manager, true);
-  }
-  else if (token_equals(event, COMMAND_SUBSCRIBE_DISPLAY_CHANGE)) {
+  } else if (token_equals(event, COMMAND_SUBSCRIBE_DISPLAY_CHANGE)) {
     bar_manager_handle_display_change(&g_bar_manager);
+  } else if (token_equals(event, COMMAND_SUBSCRIBE_SPACE_WINDOWS_CHANGE)) {
+    forced_space_windows_event();
+  } else if (token_equals(event, COMMAND_SUBSCRIBE_VOLUME_CHANGE)) {
+    forced_volume_event();
+  } else if (token_equals(event, COMMAND_SUBSCRIBE_MEDIA_CHANGE)) {
+    forced_media_change_event();
+  } else if (token_equals(event, COMMAND_SUBSCRIBE_WIFI_CHANGE)) {
+    forced_network_event();
+  } else if (token_equals(event, COMMAND_SUBSCRIBE_POWER_SOURCE_CHANGE)) {
+    forced_power_event();
   } else {
     bar_manager_custom_events_trigger(&g_bar_manager, event.text, &env_vars);
   }
@@ -183,8 +197,8 @@ static void handle_domain_add(FILE* rsp, struct token domain, char* message) {
                  command.text                                           );
   }
 
-  if (!bar_item_set_position(bar_item, position.text)
-      && bar_item->type != BAR_COMPONENT_GROUP) {
+  if (bar_item->type != BAR_COMPONENT_GROUP &&
+      !bar_item_set_position(bar_item, position.text)) {
     respond(rsp, "[!] Add %s: Illegal position '%s'\n", name.text,
                                                         position.text);
     bar_manager_remove_item(&g_bar_manager, bar_item);
@@ -359,7 +373,7 @@ static bool handle_domain_bar(FILE *rsp, struct token domain, char *message) {
     struct token state = get_token(&message);
     uint32_t adid = 0;
     if (token_equals(state, "current")) {
-      adid = display_arrangement(display_active_display_id());
+      adid = display_active_display_adid();
 
       if (adid > 0 && adid <= g_bar_manager.bar_count)
         needs_refresh = bar_manager_set_hidden(&g_bar_manager,
@@ -373,19 +387,41 @@ static bool handle_domain_bar(FILE *rsp, struct token domain, char *message) {
                                                                          g_bar_manager.any_bar_hidden));
   } else if (token_equals(command, PROPERTY_TOPMOST)) {
     struct token token = get_token(&message);
-    needs_refresh = bar_manager_set_topmost(&g_bar_manager,
-                                            evaluate_boolean_state(token,
-                                                                   g_bar_manager.topmost));
+    if (token_equals(token, ARGUMENT_WINDOW)) {
+      needs_refresh = bar_manager_set_topmost(&g_bar_manager,
+                                              TOPMOST_LEVEL_WINDOW,
+                                              true                 );
+    } else {
+      needs_refresh = bar_manager_set_topmost(&g_bar_manager,
+                                              TOPMOST_LEVEL_ALL,
+                                              evaluate_boolean_state(token,
+                                                                     g_bar_manager.topmost));
+    }
   } else if (token_equals(command, PROPERTY_STICKY)) {
     struct token token = get_token(&message);
     needs_refresh = bar_manager_set_sticky(&g_bar_manager,
                                            evaluate_boolean_state(token,
                                                                   g_bar_manager.sticky));
   } else if (token_equals(command, PROPERTY_DISPLAY)) {
-    struct token position = get_token(&message);
-    if (position.length > 0)
-      needs_refresh = bar_manager_set_display(&g_bar_manager,
-                                              position.text[0]);
+    struct token display = get_token(&message);
+
+    uint32_t display_pattern = 0;
+    uint32_t count;
+    char** list = token_split(display, ',', &count);
+    if (list && count > 0) {
+      for (int i = 0; i < count; i++) {
+        if (strcmp(list[i], ARGUMENT_DISPLAY_ALL) == 0) {
+          display_pattern = DISPLAY_ALL_PATTERN;
+        } else if (strcmp(list[i], ARGUMENT_DISPLAY_MAIN) == 0) {
+          display_pattern = DISPLAY_MAIN_PATTERN;
+        }
+        else {
+          display_pattern |= 1 << (strtoul(list[i], NULL, 0) - 1);
+        }
+      }
+      free(list);
+    }
+    needs_refresh = bar_manager_set_displays(&g_bar_manager, display_pattern);
   } else if (token_equals(command, PROPERTY_POSITION)) {
     struct token position = get_token(&message);
     if (position.length > 0)
@@ -539,7 +575,7 @@ static void handle_domain_order(FILE* rsp, struct token domain, char* message) {
   }
 
   bar_manager_sort(&g_bar_manager, ordering, count);
-  bar_manager_refresh(&g_bar_manager, false);
+  bar_manager_refresh(&g_bar_manager, false, false);
 }
 
 void handle_message_mach(struct mach_buffer* buffer) {
@@ -644,7 +680,6 @@ void handle_message_mach(struct mach_buffer* buffer) {
       handle_domain_push(rsp, command, rbr_msg);
       free(rbr_msg);
     } else if (token_equals(command, DOMAIN_UPDATE)) {
-      bar_manager_update_space_components(&g_bar_manager, true);
       bar_manager_update(&g_bar_manager, true);
       bar_needs_refresh = true;
     } else if (token_equals(command, DOMAIN_TRIGGER)) {
@@ -675,6 +710,29 @@ void handle_message_mach(struct mach_buffer* buffer) {
     } else if (token_equals(command, DOMAIN_EXIT)) {
       bar_manager_destroy(&g_bar_manager);
       exit(0);
+    } else if (token_equals(command, DOMAIN_HOTLOAD)) {
+      struct token token = get_token(&message);
+      hotload_set_state(evaluate_boolean_state(token, hotload_get_state()));
+    } else if (token_equals(command, DOMAIN_ADD_FONT)) {
+      struct token token = get_token(&message);
+      font_register(token_to_string(token));
+    } else if (token_equals(command, DOMAIN_RELOAD)) {
+      char* rbr_msg = get_batch_line(&message);
+      char* cur = rbr_msg;
+      struct token token = get_token(&cur);
+
+      bool reload = false;
+      if (token.length > 0 && token.text) {
+        if (!set_config_file_path(token.text)) {
+          respond(rsp, "[?] Reload: Invalid config path '%s'\n", token.text);
+        } else reload = true;
+      } else reload = true;
+      free(rbr_msg);
+
+      if (reload) {
+        struct event event = { NULL, HOTLOAD };
+        event_post(&event); 
+      }
     } else {
       char* rbr_msg = get_batch_line(&message);
       respond(rsp, "[!] Unknown domain '%s'\n", command.text);
@@ -691,7 +749,7 @@ void handle_message_mach(struct mach_buffer* buffer) {
 
   animator_lock(&g_bar_manager.animator);
   bar_manager_unfreeze(&g_bar_manager);
-  bar_manager_refresh(&g_bar_manager, false);
+  bar_manager_refresh(&g_bar_manager, false, false);
 
   if (rsp) fclose(rsp);
 
@@ -703,6 +761,6 @@ void handle_message_mach(struct mach_buffer* buffer) {
 }
 
 MACH_HANDLER(mach_message_handler) {
-  struct event *event = event_create(&g_event_loop, MACH_MESSAGE, message);
-  event_loop_post(&g_event_loop, event);
+  struct event event = { message, MACH_MESSAGE };
+  event_post(&event);
 }
