@@ -30,6 +30,10 @@
 
 #define DOMAIN_EXIT                            "--exit"
 
+#define DOMAIN_HOTLOAD                         "--hotload"
+#define DOMAIN_RELOAD                          "--reload"
+#define DOMAIN_ADD_FONT                        "--load-font"
+
 #define SUB_DOMAIN_ICON                        "icon"
 #define SUB_DOMAIN_LABEL                       "label"
 #define SUB_DOMAIN_BACKGROUND                  "background"
@@ -40,6 +44,11 @@
 #define SUB_DOMAIN_IMAGE                       "image"
 #define SUB_DOMAIN_KNOB                        "knob"
 #define SUB_DOMAIN_SLIDER                      "slider"
+#define SUB_DOMAIN_FONT                        "font"
+#define SUB_DOMAIN_COLOR                       "color"
+#define SUB_DOMAIN_BORDER_COLOR                "border_color"
+#define SUB_DOMAIN_HIGHLIGHT_COLOR             "highlight_color"
+#define SUB_DOMAIN_FILL_COLOR                  "fill_color"
 
 #define PROPERTY_FONT                          "font"
 #define PROPERTY_COLOR                         "color"
@@ -59,6 +68,19 @@
 #define PROPERTY_DISTANCE                      "distance"
 #define PROPERTY_ANGLE                         "angle"
 #define PROPERTY_SCALE                         "scale"
+#define PROPERTY_STRING                        "string"
+#define PROPERTY_SCROLL_TEXTS                  "scroll_texts"
+#define PROPERTY_SCROLL_DURATION               "scroll_duration"
+
+#define PROPERTY_COLOR_HEX                     "hex"
+#define PROPERTY_COLOR_ALPHA                   "alpha"
+#define PROPERTY_COLOR_RED                     "red"
+#define PROPERTY_COLOR_GREEN                   "green"
+#define PROPERTY_COLOR_BLUE                    "blue"
+
+#define PROPERTY_FONT_FAMILY                   "family"
+#define PROPERTY_FONT_STYLE                    "style"
+#define PROPERTY_FONT_SIZE                     "size"
 
 #define PROPERTY_UPDATES                       "updates"
 #define PROPERTY_POSITION                      "position"
@@ -76,11 +98,13 @@
 #define PROPERTY_IGNORE_ASSOCIATION            "ignore_association"
 #define PROPERTY_EVENT_PORT                    "mach_helper"
 #define PROPERTY_PERCENTAGE                    "percentage"
+#define PROPERTY_MAX_CHARS                     "max_chars"
 
 #define DOMAIN_BAR                             "--bar"
 #define PROPERTY_POSITION                      "position"
 #define PROPERTY_MARGIN                        "margin"
 #define PROPERTY_DISPLAY                       "display"
+#define PROPERTY_SPACE                         "space"
 #define PROPERTY_TOPMOST                       "topmost"
 #define PROPERTY_STICKY                        "sticky"
 #define PROPERTY_HIDDEN                        "hidden"
@@ -98,13 +122,18 @@
 #define COMMAND_SUBSCRIBE_SYSTEM_WOKE          "system_woke"
 #define COMMAND_SUBSCRIBE_SYSTEM_WILL_SLEEP    "system_will_sleep"
 #define COMMAND_SUBSCRIBE_VOLUME_CHANGE        "volume_change"
+#define COMMAND_SUBSCRIBE_WIFI_CHANGE          "wifi_change"
 #define COMMAND_SUBSCRIBE_BRIGHTNESS_CHANGE    "brightness_change"
 #define COMMAND_SUBSCRIBE_POWER_SOURCE_CHANGE  "power_source_change"
+#define COMMAND_SUBSCRIBE_MEDIA_CHANGE         "media_change"
 #define COMMAND_SUBSCRIBE_MOUSE_ENTERED        "mouse.entered"
 #define COMMAND_SUBSCRIBE_MOUSE_EXITED         "mouse.exited"
 #define COMMAND_SUBSCRIBE_MOUSE_CLICKED        "mouse.clicked"
+#define COMMAND_SUBSCRIBE_MOUSE_SCROLLED       "mouse.scrolled"
 #define COMMAND_SUBSCRIBE_MOUSE_ENTERED_GLOBAL "mouse.entered.global"
 #define COMMAND_SUBSCRIBE_MOUSE_EXITED_GLOBAL  "mouse.exited.global"
+#define COMMAND_SUBSCRIBE_MOUSE_SCROLLED_GLOBAL "mouse.scrolled.global"
+#define COMMAND_SUBSCRIBE_SPACE_WINDOWS_CHANGE "space_windows_change"
 
 #define DOMAIN_QUERY                           "--query"
 #define COMMAND_QUERY_DEFAULT_ITEMS            "default_menu_items"
@@ -133,9 +162,14 @@
 #define ARGUMENT_COMMON_VAL_BEFORE             "before"
 #define ARGUMENT_COMMON_VAL_AFTER              "after"
 
+#define ARGUMENT_DISPLAY_MAIN                  "main"
+#define ARGUMENT_DISPLAY_ALL                   "all"
+
 #define ARGUMENT_UPDATES_WHEN_SHOWN            "when_shown" 
 #define ARGUMENT_DYNAMIC                       "dynamic"
 
+#define ARGUMENT_WINDOW                        "window"
+
 #define POSITION_TOP          't'
 #define POSITION_BOTTOM       'b'
 #define POSITION_LEFT         'l'
@@ -145,9 +179,6 @@
 #define POSITION_CENTER_LEFT  'q'
 #define POSITION_CENTER_RIGHT 'e'
 
-#define DISPLAY_MAIN 'm'
-#define DISPLAY_ALL  'a'
-
 #define TYPE_GRAPH  "graph"
 #define TYPE_SPACE  "space"
 #define TYPE_ALIAS  "alias"
