@@ -1,54 +1,53 @@
 #include "text.h"
 #include "bar_manager.h"
 
-static CTFontRef text_create_font(char *cstring) {
-  float size = 10.0f;
-  char font_properties[2][255] = { {}, {} };
-  sscanf(cstring,
-         "%254[^:]:%254[^:]:%f",
-         font_properties[0],
-         font_properties[1],
-         &size                  );
+static void text_calculate_truncated_width(struct text* text, CFDictionaryRef attributes) {
+  if (text->max_chars > 0) {
+    uint32_t len = strlen(text->string) + 4;
+    char buffer[len];
+    memset(buffer, 0, len);
 
-  CFStringRef font_family_name = CFStringCreateWithCString(NULL,
-                                                           font_properties[0],
-                                                           kCFStringEncodingUTF8);
+    char* read = text->string;
+    char* write = buffer;
+    uint32_t counter = 0;
+    while (*read) {
+      if ((*read & 0xC0) != 0x80) counter++; 
+      if (counter > text->max_chars) {
+        break;
+      }
+      *write++ = *read++;
+    }
 
-  CFStringRef font_style_name = CFStringCreateWithCString(NULL,
-                                                          font_properties[1],
-                                                          kCFStringEncodingUTF8);
+    CFStringRef string = CFStringCreateWithCString(NULL,
+                                                   buffer,
+                                                   kCFStringEncodingUTF8);
 
-  CFNumberRef font_size = CFNumberCreate(NULL, kCFNumberFloat32Type, &size);
+    if (string) {
+      CFAttributedStringRef attr_string = CFAttributedStringCreate(NULL,
+                                                                   string,
+                                                                   attributes);
 
-  const void *keys[] = { kCTFontFamilyNameAttribute,
-                         kCTFontStyleNameAttribute,
-                         kCTFontSizeAttribute       };
+      CTLineRef line = CTLineCreateWithAttributedString(attr_string);
 
-  const void *values[] = { font_family_name, font_style_name, font_size };
-  CFDictionaryRef attributes = CFDictionaryCreate(NULL,
-                                                  keys,
-                                                  values,
-                                                  array_count(keys),
-                                                  &kCFTypeDictionaryKeyCallBacks,
-                                                  &kCFTypeDictionaryValueCallBacks);
-
-  CTFontDescriptorRef descriptor = CTFontDescriptorCreateWithAttributes(attributes);
-  CTFontRef font = CTFontCreateWithFontDescriptor(descriptor, 0.0, NULL);
-
-  CFRelease(descriptor);
-  CFRelease(attributes);
-  CFRelease(font_size);
-  CFRelease(font_style_name);
-  CFRelease(font_family_name);
-
-  return font;
+      CGRect bounds = CTLineGetBoundsWithOptions(line,
+                                              kCTLineBoundsUseGlyphPathBounds);
+      text->width = (uint32_t)(bounds.size.width + 1.5);
+      CFRelease(attr_string);
+      CFRelease(line);
+      CFRelease(string);
+    }
+  }
 }
 
 static void text_prepare_line(struct text* text) {
   const void *keys[] = { kCTFontAttributeName,
                          kCTForegroundColorFromContextAttributeName };
 
-  const void *values[] = { text->font, kCFBooleanTrue };
+  if (text->font.font_changed) {
+    font_create_ctfont(&text->font);
+    text->font.font_changed = false;
+  }
+  const void *values[] = { text->font.ct_font, kCFBooleanTrue };
   CFDictionaryRef attributes = CFDictionaryCreate(NULL,
                                                   keys,
                                                   values,
@@ -61,8 +60,8 @@ static void text_prepare_line(struct text* text) {
                                                  kCFStringEncodingUTF8);
 
   if (!string) string = CFStringCreateWithCString(NULL,
-                                                  "Warning: Malformed UTF-8 string",
-                                                  kCFStringEncodingUTF8             );
+                                          "Warning: Malformed UTF-8 string",
+                                          kCFStringEncodingUTF8             );
 
   CFAttributedStringRef attr_string = CFAttributedStringCreate(NULL,
                                                                string,
@@ -82,11 +81,14 @@ static void text_prepare_line(struct text* text) {
   text->bounds.size.height = (uint32_t) (text->bounds.size.height + 1.5);
   text->bounds.origin.x = (int32_t) (text->bounds.origin.x + 0.5);
   text->bounds.origin.y = (int32_t) (text->bounds.origin.y + 0.5);
-  text->line.color = text->highlight ? text->highlight_color : text->color;
+
+  text->width = text->bounds.size.width;
 
   CFRelease(string);
-  CFRelease(attributes);
   CFRelease(attr_string);
+
+  text_calculate_truncated_width(text, attributes);
+  CFRelease(attributes);
 }
 
 static void text_destroy_line(struct text* text) {
@@ -94,6 +96,15 @@ static void text_destroy_line(struct text* text) {
   text->line.line = NULL;
 }
 
+bool text_set_max_chars(struct text* text, uint32_t max_chars) {
+  if (text->max_chars == max_chars) return false;
+  text->max_chars = max_chars;
+  if (strlen(text->string) > text->max_chars) {
+    text_set_string(text, text->string, true);
+  }
+  return strlen(text->string) > text->max_chars;
+}
+
 bool text_set_string(struct text* text, char* string, bool forced) {
   if (!string) return false;
   if (!forced && text->string && strcmp(text->string, string) == 0) { 
@@ -107,21 +118,16 @@ bool text_set_string(struct text* text, char* string, bool forced) {
   return true;
 }
 
+void text_copy(struct text* text, struct text* source) {
+  font_set_family(&text->font, string_copy(source->font.family), true);
+  font_set_style(&text->font, string_copy(source->font.style), true);
+  font_set_size(&text->font, source->font.size);
+  text_set_string(text, string_copy(source->string), true);
+}
+
 bool text_set_font(struct text* text, char* font_string, bool forced) {
-  if (!font_string) return false;
-  if (!forced && text->font_name
-      && strcmp(text->font_name, font_string) == 0) {
-    free(font_string);
-    return false;
-  }
-
-  if (font_string != text->font_name && text->font_name)
-    free(text->font_name);
-  if (text->font) CFRelease(text->font);
-
-  text->font = text_create_font(font_string);
-  text->font_name = font_string;
-  return text_set_string(text, text->string, true);
+  bool changed = font_set(&text->font, font_string, forced);
+  return changed;
 }
 
 void text_init(struct text* text) {
@@ -132,44 +138,27 @@ void text_init(struct text* text) {
   text->padding_left = 0;
   text->padding_right = 0;
   text->y_offset = 0;
+  text->max_chars = 0;
   text->align = POSITION_LEFT;
+  text->scroll = 0.f;
+  text->scroll_duration = 100;
 
-  text->color = rgba_color_from_hex(0xffffffff);
-  text->highlight_color = rgba_color_from_hex(0xff000000);
-
-  text->font = NULL;
   text->string = string_copy("");
-  text->font_name = string_copy("Hack Nerd Font:Bold:14.0");
-  text_set_font(text, text->font_name, true);
   text_set_string(text, text->string, false);
   shadow_init(&text->shadow);
   background_init(&text->background);
-}
+  font_init(&text->font);
 
-static bool text_update_color(struct text* text) {
-  struct rgba_color target_color = text->highlight
-                                   ? text->highlight_color
-                                   : text->color;
-
-  if (text->line.color.r == target_color.r 
-      && text->line.color.g == target_color.g 
-      && text->line.color.b == target_color.b 
-      && text->line.color.a == target_color.a) {
-    return false;
-  }
-
-  text->line.color = target_color;
-  return true;
+  color_init(&text->color, 0xffffffff);
+  color_init(&text->highlight_color, 0xff000000);
 }
 
 static bool text_set_color(struct text* text, uint32_t color) {
-  text->color = rgba_color_from_hex(color);
-  return text_update_color(text);
+  return color_set_hex(&text->color, color);
 }
 
-static bool text_set_hightlight_color(struct text* text, uint32_t color) {
-  text->highlight_color = rgba_color_from_hex(color);
-  return text_update_color(text);
+static bool text_set_highlight_color(struct text* text, uint32_t color) {
+  return color_set_hex(&text->highlight_color, color);
 }
 
 static bool text_set_padding_left(struct text* text, int padding) {
@@ -190,6 +179,12 @@ static bool text_set_yoffset(struct text* text, int offset) {
   return true;
 }
 
+static bool text_set_scroll_duration(struct text* text, int duration) {
+  if (duration < 0) return false;
+  text->scroll_duration = duration;
+  return false;
+}
+
 static bool text_set_width(struct text* text, int width) {
   if (width < 0) {
     bool prev = text->has_const_width;
@@ -205,21 +200,26 @@ static bool text_set_width(struct text* text, int width) {
 
 void text_clear_pointers(struct text* text) {
   text->string = NULL;
-  text->font_name = NULL;
-  text->font = NULL;
   text->line.line = NULL;
   background_clear_pointers(&text->background);
+  font_clear_pointers(&text->font);
 }
 
 uint32_t text_get_length(struct text* text, bool override) {
   if (!text->drawing) return 0;
-  int len = text->bounds.size.width + text->padding_left + text->padding_right;
+
+  if (text->font.font_changed) {
+    text_set_string(text, text->string, true);
+  }
+
+  int len = text->width + text->padding_left + text->padding_right;
   if ((!text->has_const_width || override)
       && text->background.enabled
-      && text->background.image.enabled
-      && text->background.image.bounds.size.width > len
-      && !text->has_const_width                        ) {
-    return text->background.image.bounds.size.width;
+      && text->background.image.enabled) {
+    CGSize image_size = image_get_size(&text->background.image);
+    if (image_size.width > len) {
+      return image_size.width;
+    }
   }
 
   if (text->has_const_width && !override) return text->custom_width;
@@ -232,9 +232,9 @@ uint32_t text_get_height(struct text* text) {
 
 void text_destroy(struct text* text) {
   background_destroy(&text->background);
+  font_destroy(&text->font);
+
   if (text->string) free(text->string);
-  if (text->font_name) free(text->font_name);
-  if (text->font) CFRelease(text->font);
   text_destroy_line(text);
   text_clear_pointers(text);
 }
@@ -265,11 +265,72 @@ void text_calculate_bounds(struct text* text, uint32_t x, uint32_t y) {
   }
 }
 
+bool text_set_scroll(struct text* text, float scroll) {
+  if (text->scroll == scroll) return false;
+  text->scroll = scroll;
+  return true;
+}
+
+bool text_animate_scroll(struct text* text) {
+  if (text->max_chars == 0) return false;
+  if (text->scroll != 0) return false;
+  if (text->has_const_width && text->custom_width < text->width) return false;
+  if (text->width == 0 || text->width == text->bounds.size.width) return false;
+
+  g_bar_manager.animator.duration = text->scroll_duration
+                                    * (text->bounds.size.width / text->width);
+  g_bar_manager.animator.interp_function = INTERP_FUNCTION_LINEAR;
+
+  bool needs_refresh = false;
+  ANIMATE_FLOAT(text_set_scroll,
+                text,
+                text->scroll,
+                max(text->bounds.size.width, 0));
+
+  struct animation* animation = animation_create();
+  float initial_value = text->scroll;
+  float final_value = -max(text->width, 0);
+
+  animation_setup(animation,
+                  (void*)text,
+                  (bool (*)(void*, int))text_set_scroll,
+                  *(int*)&initial_value,
+                  *(int*)&final_value,
+                  0,
+                  INTERP_FUNCTION_LINEAR );
+  animation->as_float = true;
+  animator_add(&g_bar_manager.animator, animation);
+
+  g_bar_manager.animator.duration = text->scroll_duration;
+  ANIMATE_FLOAT(text_set_scroll, text, text->scroll, 0);
+
+  g_bar_manager.animator.duration = 0;
+  g_bar_manager.animator.interp_function = '\0';
+
+  return needs_refresh;
+}
+
 void text_draw(struct text* text, CGContextRef context) {
   if (!text->drawing) return;
   if (text->background.enabled)
     background_draw(&text->background, context);
 
+  CGContextSaveGState(context);
+  if (text->max_chars > 0) {
+    CGMutablePathRef path = CGPathCreateMutable();
+    CGRect bounds = text->bounds;
+    bounds.size.width = text->width;
+    bounds.origin.x += text->padding_left;
+    bounds.origin.y = -9999.f;
+    bounds.size.height = 2.f*9999.f;
+
+    CGPathAddRect(path, NULL, bounds);
+
+    CGContextAddPath(context, path);
+    CGContextClip(context);
+    CFRelease(path);
+  }
+
   if (text->shadow.enabled) {
     CGContextSetRGBFillColor(context,
                              text->shadow.color.r,
@@ -284,16 +345,15 @@ void text_draw(struct text* text, CGContextRef context) {
     CTLineDraw(text->line.line, context);
   }
 
-  CGContextSetRGBFillColor(context,
-                           text->line.color.r,
-                           text->line.color.g,
-                           text->line.color.b,
-                           text->line.color.a );
+  struct color color = text->highlight ? text->highlight_color : text->color;
+  CGContextSetRGBFillColor(context, color.r, color.g, color.b, color.a);
 
   CGContextSetTextPosition(context,
-                           text->bounds.origin.x + text->padding_left,
-                           text->bounds.origin.y + text->y_offset     );
+                           text->bounds.origin.x + text->padding_left
+                           - text->scroll,
+                           text->bounds.origin.y + text->y_offset    );
   CTLineDraw(text->line.line, context);
+  CGContextRestoreGState(context);
 }
 
 void text_serialize(struct text* text, char* indent, FILE* rsp) {
@@ -327,21 +387,21 @@ void text_serialize(struct text* text, char* indent, FILE* rsp) {
                "%s\"padding_left\": %d,\n"
                "%s\"padding_right\": %d,\n"
                "%s\"y_offset\": %d,\n"
-               "%s\"font\": \"%s\",\n"
+               "%s\"font\": \"%s:%s:%.2f\",\n"
                "%s\"width\": %d,\n"
                "%s\"align\": \"%s\",\n"
                "%s\"background\": {\n",
                indent, text->string,
                indent, format_bool(text->drawing),
                indent, format_bool(text->highlight),
-               indent, hex_from_rgba_color(text->color),
-               indent, hex_from_rgba_color(text->highlight_color),
+               indent, text->color.hex,
+               indent, text->highlight_color.hex,
                indent, text->padding_left,
                indent, text->padding_right,
                indent, text->y_offset,
-               indent, text->font_name,
+               indent, text->font.family, text->font.style, text->font.size,
                indent, text->custom_width,
-               indent, align, indent                              );
+               indent, align, indent                                        );
 
   char deeper_indent[strlen(indent) + 2];
   snprintf(deeper_indent, strlen(indent) + 2, "%s\t", indent);
@@ -358,43 +418,51 @@ bool text_parse_sub_domain(struct text* text, FILE* rsp, struct token property,
     struct token token = get_token(&message);
     ANIMATE_BYTES(text_set_color,
                   text,
-                  hex_from_rgba_color(text->color),
-                  token_to_int(token)              );
+                  text->color.hex,
+                  token_to_int(token));
   }
   else if (token_equals(property, PROPERTY_HIGHLIGHT)) {
     bool highlight = evaluate_boolean_state(get_token(&message),
                                              text->highlight    );
     if (g_bar_manager.animator.duration > 0) {
       if (text->highlight && !highlight) {
-        uint32_t target = hex_from_rgba_color(text->color);
-        text_set_color(text, hex_from_rgba_color(text->highlight_color));
+        animator_cancel(&g_bar_manager.animator,
+                        text,
+                        (animator_function*)text_set_color);
+
+        uint32_t target = text->color.hex;
+        text_set_color(text, text->highlight_color.hex);
 
         ANIMATE_BYTES(text_set_color,
                       text,
-                      hex_from_rgba_color(text->color),
-                      target                           );
+                      text->color.hex,
+                      target          );
       }
       else if (!text->highlight && highlight) {
-        uint32_t target = hex_from_rgba_color(text->highlight_color);
-        text_set_hightlight_color(text, hex_from_rgba_color(text->color));
+        animator_cancel(&g_bar_manager.animator,
+                        text,
+                        (animator_function*)text_set_highlight_color);
 
-        ANIMATE_BYTES(text_set_hightlight_color,
+        uint32_t target = text->highlight_color.hex;
+        text_set_highlight_color(text, text->color.hex);
+
+        ANIMATE_BYTES(text_set_highlight_color,
                       text,
-                      hex_from_rgba_color(text->highlight_color),
-                      target                                     );
+                      text->highlight_color.hex,
+                      target                    );
       }
     }
 
+    needs_refresh = text->highlight != highlight;
     text->highlight = highlight;
-    needs_refresh = text_update_color(text);
   } else if (token_equals(property, PROPERTY_FONT))
     needs_refresh = text_set_font(text, string_copy(message), false);
   else if (token_equals(property, PROPERTY_HIGHLIGHT_COLOR)) {
     struct token token = get_token(&message);
-    ANIMATE_BYTES(text_set_hightlight_color,
+    ANIMATE_BYTES(text_set_highlight_color,
                   text,
-                  hex_from_rgba_color(text->highlight_color),
-                  token_to_int(token)                        );
+                  text->highlight_color.hex,
+                  token_to_int(token)       );
 
   } else if (token_equals(property, PROPERTY_PADDING_LEFT)) {
     struct token token = get_token(&message);
@@ -417,6 +485,9 @@ bool text_parse_sub_domain(struct text* text, FILE* rsp, struct token property,
             text->y_offset,
             token_to_int(token));
 
+  } else if (token_equals(property, PROPERTY_SCROLL_DURATION)) {
+    struct token token = get_token(&message);
+    text_set_scroll_duration(text, token_to_int(token));
   } else if (token_equals(property, PROPERTY_WIDTH)) {
     struct token token = get_token(&message);
     if (token_equals(token, ARGUMENT_DYNAMIC)) {
@@ -431,7 +502,7 @@ bool text_parse_sub_domain(struct text* text, FILE* rsp, struct token property,
                       (bool (*)(void*, int))&text_set_width,
                       text->custom_width,
                       -1,
-                      1,
+                      0,
                       INTERP_FUNCTION_LINEAR               );
       animator_add(&g_bar_manager.animator, animation);
     }
@@ -449,19 +520,60 @@ bool text_parse_sub_domain(struct text* text, FILE* rsp, struct token property,
     char prev = text->align;
     text->align = get_token(&message).text[0];
     return prev != text->align;
-  } 
+  } else if (token_equals(property, PROPERTY_STRING)) {
+    uint32_t pre_width = text_get_length(text, false);
+    bool changed = text_set_string(text,
+                                   token_to_string(get_token(&message)),
+                                   false                                );
+
+    if (changed
+        && g_bar_manager.animator.duration > 0) {
+      uint32_t post_width = text_get_length(text, false);
+      if (post_width != pre_width) {
+        text_set_width(text, pre_width);
+        ANIMATE(text_set_width, text, pre_width, post_width);
+
+        struct animation* animation = animation_create();
+        animation_setup(animation,
+                        text,
+                        (bool (*)(void*, int))&text_set_width,
+                        text->custom_width,
+                        -1,
+                        0,
+                        INTERP_FUNCTION_LINEAR               );
+        animator_add(&g_bar_manager.animator, animation);
+      }
+    }
+
+    return changed;
+  } else if (token_equals(property, PROPERTY_MAX_CHARS)) {
+    return text_set_max_chars(text, token_to_int(get_token(&message)));
+  }
   else {
-    struct key_value_pair key_value_pair = get_key_value_pair(property.text, '.');
+    struct key_value_pair key_value_pair = get_key_value_pair(property.text,
+                                                              '.'           );
     if (key_value_pair.key && key_value_pair.value) {
       struct token subdom = { key_value_pair.key, strlen(key_value_pair.key) };
-      struct token entry = { key_value_pair.value, strlen(key_value_pair.value) };
+      struct token entry = { key_value_pair.value,
+                             strlen(key_value_pair.value) };
       if (token_equals(subdom, SUB_DOMAIN_BACKGROUND))
-        return background_parse_sub_domain(&text->background, rsp, entry, message);
+        return background_parse_sub_domain(&text->background,
+                                           rsp,
+                                           entry,
+                                           message           );
       else if (token_equals(subdom, SUB_DOMAIN_SHADOW))
         return shadow_parse_sub_domain(&text->shadow, rsp, entry, message);
-      else {
+      else if (token_equals(subdom, SUB_DOMAIN_FONT))
+        return font_parse_sub_domain(&text->font, rsp, entry, message);
+      else if (token_equals(subdom, SUB_DOMAIN_COLOR))
+        return color_parse_sub_domain(&text->color, rsp, entry, message);
+      else if (token_equals(subdom, SUB_DOMAIN_HIGHLIGHT_COLOR))
+        return color_parse_sub_domain(&text->highlight_color,
+                                      rsp,
+                                      entry,
+                                      message);
+      else
         respond(rsp, "[!] Text: Invalid subdomain '%s' \n", subdom.text);
-      }
     }
     else {
       respond(rsp, "[!] Text: Invalid property '%s'\n", property.text);
