@@ -1,5 +1,5 @@
-#include "Carbon/Carbon.h"
 #include "CoreAudio/CoreAudio.h"
 #include <CoreAudio/AudioHardware.h>
 
+void forced_volume_event();
 void begin_receiving_volume_events();
